#include <iostream>
#include <wiringPi.h>
#include "binary.h"

static const int DS {0};
static const int STCP {1};
static const int SHCP {2};

u_char round[] = {1_b, 10_b, 100_b, 1000_b, 1.0000_b, 10.0000_b};
u_char digits[] = {11.1111_b, 110_b, 101.1011_b, 100.1111_b, 110.0110_b,
                   110.1101_b, 111.1101_b, 111_b, 111.1111_b, 110.1111_b,
                   111.0111_b, 111.1100_b, 11.1001_b, 101.1110_b, 111.1001_b, 111.0001_b};

void init()
{
    for (int pin: {DS, STCP, SHCP}) {
        pinMode(pin, OUTPUT);
        digitalWrite(pin, LOW);
    }
}

inline void pulse(int pin)
{
    digitalWrite(pin, HIGH);
    delay(1);
    digitalWrite(pin, LOW);
}

inline void sipo(u_char data)
{
    for (int i {0}; i != 8; ++i) {
        digitalWrite(DS, (0x80 & (data << i)) ? HIGH : LOW);
        pulse(SHCP);
    }
    pulse(STCP);
}

void goRound()
{
    for (u_char d: round) {
        sipo(d);
        delay(40);
    }
}

void showDigits()
{
    bool showPoint {false};
    for (u_char d: digits) {
        sipo(d | (showPoint ? 1000.0000_b : 0));
        showPoint = !showPoint;
        delay(500);
    }
}

int main()
{
    if (wiringPiSetup() == -1) return 1;
    init();
    while (true) {
        goRound();
        goRound();
        showDigits();
    }
    return 0;
}
